#include <stdio.h>
#include <stdlib.h>
#include <malloc.h>
#include <string.h>
#define WIN32_LEAN_AND_MEAN
#include <windows.h>

int insert_word(char * new_word);
void report(void);
void init_gen(void);

/*
 * Word counter, client program
 * Jari Komppa 1999
 */


FILE * infile;
char * wordbuf;
#define MAX_WORD_LEN 20
int word_count=0, max_words=0;

/*
 * Turn characters downcase, return 0 for non-alphanumeric chars for word break.
 */
char dncase(int data) {
  if (data>='a' && data<='z')
    return data;
  if (data>='0' && data<='9')
    return data;
  if (data>='A' && data<='Z')
    return data+('a'-'A');
  return 0;
}

/*
 * Scan the text buffer for the next word
 */
char * parse_new_word(void) {
  int word_len;
  do {
    word_len=0;
    int ch=0;
    while ((ch=getc(infile))!=EOF) {
      wordbuf[word_len]=dncase(ch);
      if (wordbuf[word_len]==0) break;
      word_len++;
    }
    if ((ch==EOF) && (word_len==0))
      return NULL;
  } while ((word_len>MAX_WORD_LEN) || (word_len==0));
  return wordbuf;
}

/*
 * Init by opening the input file and counting words.
 * (We should also have the input file in disk cache now)
 */
void init(void) {
  setbuf(stdout,NULL);
  infile=fopen("cpp.txt","r");
  wordbuf=(char*)malloc(2048);
  while (parse_new_word()!=NULL) max_words++;
  fseek(infile,SEEK_SET,0);
  init_gen();
}

/*
 * Output by-second update
 */
void second_update(void) {
  int lin; 
  static int secs=0;
  lin=word_count/(max_words/60);
  printf("%3ds %3dkword %3d%% ",secs,word_count/1000,(word_count*100)/max_words);
  while (lin-->0) printf("=");
  if ((word_count/(max_words/30))&1) printf("-");
  printf("\n");
  secs++;
}

/*
 * Parse through the file
 */
void parse(void) {
  int last_tick=0;
  while (insert_word(parse_new_word())!=0) {
    word_count++;
    if (last_tick<GetTickCount()) {
      last_tick=GetTickCount()+1000;
      second_update();
    }
  }
}

/******************************************************
 * Report generation helper functions
 * (simple insertion sort)
 */

typedef struct insertSortList {
	insertSortList * next;
	char * data;
	int hits;
} SORTITEM;
 
SORTITEM * sroot;

void insert_sort(ITEM * i) {
	SORTITEM * n;
	n=(SORTITEM*)malloc(sizeof(SORTITEM));
	n->data=i->data;
	n->hits=i->hits;
	n->next=NULL;
	if (sroot==NULL) {
		sroot=n;
		return;
	}
	if (sroot->hits<n->hits) {
		n->next=sroot;
		sroot=n;
		return;
	}
	SORTITEM * r=sroot, * l=sroot;
	while (r->hits>=n->hits && r->next!=NULL) {
		l=r;
		r=r->next;
	}
	if (r->next==NULL) {
		l=r;
	}
	n->next=l->next;
	l->next=n;
}

/*
 * Report generation helper functions
 ******************************************************/



/*
 * Well.. main.
 */
void main(void) {
  int start_tick,end_tick;
  init();			// Init
  start_tick=GetTickCount();	// Set start tick
  parse();			// Parse through the input
  end_tick=GetTickCount();	// Get end tick.
  printf("\nProcess time: %3.3f seconds, %d words (%3.3f words per second)\n",(end_tick-start_tick)/1000.0,word_count,word_count/((end_tick-start_tick)/1000.0));
  report();			// Output report (sorted list)
}
